```wmd-options
language = es
title_i18n = wmarkdown_title_lists
title_text = Listas - WMarkDown
```

<!-- [[wmd]] -->

### Listas

Los listas en WMarkDown se basan en una mezcla entre lo que son las listas crudas de MarkDown así como en las listas de MediaWiki, con un concepto de personalización para determinar si un elemento está contraído o no, permitiendo así controlar la visualización inicial de la lista.

> [!!] Es importante mencionar que la visualización de los elementos compactados depende plenamente de que JavaScript esté habilitado en el navegador pues por defecto no se mostrarán y las acciones dinámicas dependerán plenamente de que éste esté habilitado.

Partiendo de la idea de MarkDown para la gestión de Listas y sus marcas alternativas se creó un subprotocolo que condiciona qué marca se usar'para qué fin pues no todas las marcas funcionan igual, además, éste sistema altera la guía de buenas prácticas de MarkDown pues éste dice de usar siempre la misma marca en la misma lista o nivel de la misma, mientras que aquí se podrá ir alternando según intereses. Las marcas con sus significados son los siguientes:

- **&ast;**: El asterisco nos indica una lista desordenada estándar, sin posibilidad de desplegar o compactar.
- **&plus;**: El signo más nos dice que será una lista desordenada y que ésta estará desplegada por defecto con opción a compactar.
- **&minus;**: El signo menos nos dice que será una lista desordenada y que ésta estará compactada por defecto con opción a desplegar.
- **&num;**: El signo comodín, o sostenido o numérico indica que será una lista ordenada mediante números que parten del valor "1".
- **1.**: Cualquier número seguido de un punto nos indica una lista ordenada numérica cuyo valor inicial será el del primer elemento puesto.
- **a.**: Cualquier letra, a excepción de la letra **i** latina, seguida de un punto nos indica que será una lista ordenada alfabéticamente. Éste puede ser en mayúsculas para indicar que las letras sean mayúsculas, o minúsculas para lo contrario.
- **i.**: Un número romano seguido de un punto nos indica que la lista será ordenada con números romanos. Si éstos están en mayúsculas aparecerán en mayúsculas, y si son en minúsculas, éstos aparecerán en minúsculas.

> [!!] En el caso de los elementos ordenados, éstos seguirán las pautas única y exclusivamente del primer elemento dado. Si por el medio se hacen saltos, se repiten o se mantienen valores, así como si se integran marcas de listas desordenadas, éstas continuarán su orden interpretando corrección del usuario.

Los tipos de listas que hay se pueden ver reflejadas en los siguientes ejemplos:

```wmd-examples

Ejemplo de lista desordenada:

* Elemento 1.
* Elemento 2.
* Elemento 3.
* Elemento 4.
* Elemento N.

Ejemplo de lista ordenada numérica:

# Elemento 1.
# Elemento 2.
# Elemento 3.
# Elemento 4.
# Elemento N.

Ejemplo de lista ordenada con inicio en otro valor que no sea uno.

4. Elemento 1.
5. Elemento 2.
6. Elemento 3.
7. Elemento 4.
8. Elemento N.

La lista anterior sería igual que hacer lo siguiente:

4. Elemento 1.
4. Elemento 2.
4. Elemento 3.
4. Elemento 4.
4. Elemento N.

O incluso lo siguiente:

4. Elemento 1.
# Elemento 2.
# Elemento 3.
# Elemento 4.
# Elemento N.

```

Las listas se pueden anidar, lo que hacen que crezcan en árbol. Para ello tenemos tanto la filosofía base de MarkDown mediante espacios y/o tabulaciones; como la filosofía de MediaWiki mediante el conjunto de marcas que determinan el nivel del elemento dentro de la lista.

```wmd-examples

Listas anidadas por el método de MarkDown.

* Elemento 1.
* Elemento 2.
    * Elemento 2-1.
    * Elemento 2-2.
        * Elemento 2-2-1.
    * Elemento 2-3.
* Elemento 3.
* Elemento 4.
    * Elemento 4-1.
* Elemento N.

Listas anidadas por el método de MediaWiki.

* Elemento 1.
* Elemento 2.
** Elemento 2-1.
** Elemento 2-2.
*** Elemento 2-2-1.
** Elemento 2-3.
* Elemento 3.
* Elemento 4.
** Elemento 4-1.
* Elemento N.

Ambas darán el mismo resultado.

```

Dentro de las listas anidadas hay dos marcas que permiten controlar si salen compactadas o desplegadas, lo que determina también que serían compactables o desplegables por el usuario final.

```wmd-examples

* Elemento 1. Elemento desplegado pero no desplegable o compactable.
    * Elemento 1-1.
    * Elemento 1-2.
    * Elemento 1-3.
- Elemento 2. Elemento desplegable que por defecto está plegado.
    * Elemento 2-1.
    + Elemento 2-2. Elemento desplegable, desplegado por defecto.
        * Elemento 2-2-1.
    - Elemento 2-3. Elemento desplegable que por defecto está plegado.
        * Elemento 2-3-1.
* Elemento 3.
+ Elemento 4. Elemento desplegable que por defecto está desplegado.
    * Elemento 4-1.
* Elemento N.

```

> [!!] Como podemos observar, la guía de las buenas prácticas de MarkDown cara las listas, con este concepto se rompe.

También se pueden anidar listas ordenadas y desordenadas entre sí.

```wmd-examples

Ejemplo en filosofía MarkDown.

* Elemento 1.
* Elemento 2.
    # Elemento 2-1.
    # Elemento 2-2.
        * Elemento 2-2-1.
    # Elemento 2-3.
* Elemento 3.
* Elemento 4.
    # Elemento 4-1.
* Elemento N.

Ejemplo en filosofía MediaWiki.

* Elemento 1.
* Elemento 2.
*# Elemento 2-1.
*# Elemento 2-2.
*#* Elemento 2-2-1.
*# Elemento 2-3.
* Elemento 3.
* Elemento 4.
*# Elemento 4-1.
* Elemento N.

```

Mediante la filosofía de poder cambiar la marca tanto en una lista ordenada como desordenada, a excepción del primer valor de las listas ordenadas, en una lista ordenada se pueden establecer elementos plegados y desplegados.

```wmd-examples

4. Elemento 1.
+ Elemento 2.
    # Elemento 2-1.
    # Elemento 2-2.
    # Elemento 2-3.
* Elemento 3.
    # Elemento 3-1.
    # Elemento 3-2.
    # Elemento 3-3.
- Elemento 4.
    # Elemento 4-1.
    # Elemento 4-2.
    # Elemento 4-3.
* Elemento N.
    # Elemento N-1.
    # Elemento N-2.
    # Elemento N-3.

```

<!-- [[wmd]] -->